#import <Foundation/Foundation.h>
#import <AppKit/Appkit.h>
#import <CoreGraphics/CoreGraphics.h>

#include <u.h>
#include <libc.h>

AUTOFRAMEWORK(Foundation)
AUTOFRAMEWORK(AppKit)
AUTOFRAMEWORK(CoreGraphics)
AUTOFRAMEWORK(Quartz)

void
usage(void)
{
	fprint(2, "usage: resample [-i] [-x xsize] [-y ysize] [imagefile]\n");
	fprint(2, "\tuse -i to generated iOS AppIcon sizes, or\n");
	fprint(2, "\twhere size is an integer or a percentage in the form 25%%\n");
	exits("usage");
}

#define APPICONS 8
int appicon_table[APPICONS] = {
	29, 40, 58, 60, 76, 80, 120, 152,
};

int iosflag;

int
getint(char *s, int *percent)
{
	if(s == nil)
		usage();
	*percent = (s[strlen(s)-1] == '%');
	if(*s == '+')
		return atoi(s+1);
	if(*s == '-')
		return -atoi(s+1);
	return atoi(s);
}

NSString*
newfilepath(NSString *path, int xsize, int ysize)
{
	NSString *ext, *filename, *np;
	ext = [path pathExtension];
	filename = [path stringByDeletingPathExtension];
	np = [NSString stringWithFormat:@"%@-%dx%d.%@", filename, xsize, ysize, ext];
	return np;
}

CIImage *
resample(CGImageRef m, int xsize, int ysize)
{
	CIImage *ciim, *img;
	CGAffineTransform ctm;
	CGSize msize, dsize;
	CGFloat xs, ys;
	NSDictionary *props;

	props = [NSDictionary dictionaryWithObject:(id)CGImageGetColorSpace(m) forKey:kCIImageColorSpace];
	msize = CGSizeMake(CGImageGetWidth(m), CGImageGetHeight(m));
	dsize = CGSizeMake(xsize, ysize);

	xs = dsize.width/msize.width;
	ys = dsize.height/msize.height;

	ctm = CGAffineTransformMakeScale(xs, ys);
	ciim = [CIImage imageWithCGImage:m options:props];
	img = [ciim imageByApplyingTransform:ctm];

	return img;
}

NSUInteger
writeimage(CGImageRef m, CFStringRef imgtype, NSString *path)
{
	NSUInteger r = 0;
	NSMutableData *data;
	CGImageDestinationRef dest = nil;

	if(m == NULL)
		return 0;

	data = [NSMutableData data];
	dest = CGImageDestinationCreateWithData((__bridge CFMutableDataRef)data, imgtype, 1, NULL);

	if(dest) {
        CGImageDestinationAddImage(dest, m, NULL);
		if(CGImageDestinationFinalize(dest)){
			if([path isEqualToString:@"<stdin>"]){
				NSFileHandle *fd;
				fd = [NSFileHandle fileHandleWithStandardOutput];		
				[fd writeData:data];
				r = [data length];
			} else {
				if([data writeToFile:path atomically:YES])
					r = [data length];
			}
		}

		CFRelease(dest);
	}
	return r;
}

void
main(int argc, char * argv[])
{
	int xsize, ysize, xpercent, ypercent;
	char *file;

	xsize = ysize = 0;
	xpercent = ypercent = 0;
	
	ARGBEGIN{
	case 'a':	/* compatibility; equivalent to just -x or -y */
		if(xsize != 0 || ysize != 0)
			usage();
		xsize = getint(ARGF(), &xpercent);
		if(xsize <= 0)
			usage();
		ysize = xsize;
		ypercent = xpercent;
		break;
	case 'x':
		if(xsize != 0)
			usage();
		xsize = getint(ARGF(), &xpercent);
		if(xsize <= 0)
			usage();
		break;
	case 'y':
		if(ysize != 0)
			usage();
		ysize = getint(ARGF(), &ypercent);
		if(ysize <= 0)
			usage();
		break;
	case 'i':	iosflag++; break;
	default:
		usage();
	}ARGEND

	if((xsize == 0 && ysize == 0) && !iosflag)
		usage();

	file = "<stdin>";
	if(argc > 1 || (argc == 0 && iosflag))
		usage();
	else if(argc == 1){
		file = argv[0];
	}

	@autoreleasepool {
		NSString *path;
		NSFileHandle *fd;
		NSData *data;
		NSBitmapImageRep *rep;
		CIImage *m;
		CGImageRef src;
		CGImageSourceRef isrc;

		if(argc == 1)
			fd = [NSFileHandle fileHandleForReadingAtPath:[NSString stringWithCString:file encoding:NSUTF8StringEncoding]];
		else
			fd = [NSFileHandle fileHandleWithStandardInput];

		data = [fd readDataToEndOfFile];
		assert(data);
		isrc = CGImageSourceCreateWithData((__bridge CFDataRef)data, NULL);

		src = CGImageSourceCreateImageAtIndex(isrc, 0, NULL);
		assert(src);

		path = [NSString stringWithCString:file encoding:NSUTF8StringEncoding];

		if(iosflag) {
			int i, sz;
			NSString *np;
			for(i = 0; i < APPICONS; i++) {
				sz = appicon_table[i];
				m = resample(src, sz, sz);
				assert(m);
				np = newfilepath(path, sz, sz);
				rep = [[NSBitmapImageRep alloc] initWithCIImage:m];
				writeimage(rep.CGImage, CGImageSourceGetType(isrc), np);
				[rep release];
			}
		} else {
			NSString *np;
			if(xpercent)
				xsize = CGImageGetWidth(src) * xsize/100;
			if(ypercent)
				ysize = CGImageGetHeight(src) * ysize/100;
			if(ysize == 0)
				ysize = xsize * CGImageGetHeight(src) / CGImageGetWidth(src);
			if(xsize == 0)
				xsize = ysize * CGImageGetWidth(src) / CGImageGetHeight(src);

			m = resample(src, xsize, ysize);
			assert(m);
			np = newfilepath(path, xsize, ysize);
			rep = [[NSBitmapImageRep alloc] initWithCIImage:m];
			writeimage(rep.CGImage, CGImageSourceGetType(isrc), np);
			[rep release];
		}
		CFRelease(src);
		CFRelease(isrc);
	}

	exits(nil);
}
